#include <PalmOS.h>				
#include "jedi.h"

/*********************************************************************/
/*********************************************************************/
static Err StartApplication(void)
{
  FrmGotoForm(JediForm);
  SysRandom(TimGetTicks());
  return 0;
}

/*********************************************************************/
/*********************************************************************/
static void StopApplication(void)
{
  FrmCloseAllForms();
}

/*********************************************************************/
/*********************************************************************/
static Int16 RandInt(Int16 MaxNo)
{
  // returns 0 <= int <= MaxNo-1
  return(SysRandom(0) % MaxNo);
}

/*********************************************************************/
/*********************************************************************/
static void SetFieldTextFromStr(UInt16 fieldID, Char * strP)
{
  MemHandle txtH, oldTxtH;
  FormPtr frm = FrmGetActiveForm();
  FieldPtr fldP;
	
  txtH = MemHandleNew(StrLen(strP) + 1);
  if (txtH) {
    StrCopy(MemHandleLock(txtH), strP);
    MemHandleUnlock(txtH);
    fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
    oldTxtH = FldGetTextHandle(fldP);
    FldSetTextHandle(fldP, txtH);
    FldDrawField(fldP);
    if (oldTxtH)
      MemHandleFree(oldTxtH);
  }
}

/*********************************************************************/
/*********************************************************************/
static Boolean JediFormHandleEvent(EventPtr event)
{
  static Int16 oldrnd = 0;
  Boolean handled = false;
  Int16	rnd;
  MemHandle h;
  Int8 *s;
  
  switch (event->eType) {
  case ctlSelectEvent:  // A control button was pressed and released.
    if (event->data.ctlEnter.controlID==WisdomBtn)
      {
	rnd = 3000+RandInt(63);
	while(rnd==oldrnd) rnd = 3000+RandInt(63);
	oldrnd=rnd;
	h = DmGetResource('tSTR', rnd);
	s = MemHandleLock(h);
	SetFieldTextFromStr(JediField, s);
	MemHandleUnlock(h);
	DmReleaseResource(h);
	handled = true;
      }
    break;

  case frmOpenEvent:	
    FrmDrawForm(FrmGetActiveForm());
    oldrnd = 3000+RandInt(63);
    h = DmGetResource('tSTR', oldrnd);
    s = MemHandleLock(h);
    SetFieldTextFromStr(JediField, s);
    MemHandleUnlock(h);
    DmReleaseResource(h);
    break;

  case menuEvent:
    if (event->data.menu.itemID==5002) {
      FrmHelp(5000);
      handled = true;
    }
    break;
  }			
  return(handled);
}

/*********************************************************************/
/*********************************************************************/
static Boolean ApplicationHandleEvent(EventPtr event)
{
  FormPtr frm;
  Int16	formId;
  Boolean handled = false;

  if (event->eType == frmLoadEvent) {
    // Load the form resource specified in the event then activate the form.
    formId = event->data.frmLoad.formID;
    frm = FrmInitForm(formId);
    FrmSetActiveForm(frm);

    // Set the event handler for the form.  The handler of the currently 
    // active form is called by FrmDispatchEvent each time it receives an event
    switch (formId) {
    case JediForm:
      FrmSetEventHandler(frm, JediFormHandleEvent);
      handled = true;
      break;
    }
  }
  return handled;
}

/*********************************************************************/
/*********************************************************************/
static void EventLoop(void)
{
  EventType event;
  UInt16 error;
	
  do {
    EvtGetEvent(&event, evtWaitForever);
    if (!SysHandleEvent(&event))
      if (!MenuHandleEvent(0, &event, &error))
	if (!ApplicationHandleEvent(&event))
	  FrmDispatchEvent(&event);
  }
  while (event.eType != appStopEvent);
}

/*********************************************************************/
/*********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
  Err error = 0;		
  if (cmd == sysAppLaunchCmdNormalLaunch) {
    if ((error = StartApplication()) == 0) {   
      EventLoop();
      StopApplication();
    }
  }
  return error;
}
